#!/usr/bin/env python3

import math
import time

# https://github.com/srmainwaring/python-ignition
from ignition.msgs.header_pb2 import Header
from ignition.msgs.pose_pb2 import Pose
from ignition.msgs.quaternion_pb2 import Quaternion
from ignition.msgs.vector3d_pb2 import Vector3d
from ignition.transport import Node

# https://github.com/srmainwaring/python-ignition
import ignition.math

# Settings
world_name = "empty"
model_name = "iris"

timeout_ms = 10
update_rate_hz = 30.0

# offsets - The ArduPilot z position may be zeroed to the ground plane,
#           Gazebo will set pose for the base link geometric centre
pose_offset_x = 0.0
pose_offset_y = 0.0
pose_offset_z = 0.17


def main():

    connection_string = "udp:127.0.0.1:14550"

    # Connect to the Vehicle.
    print("Connecting to vehicle on: {}".format(connection_string))

    try:
        # create a transport node
        node = Node()

        # set service details
        service = "/world/{}/set_pose".format(world_name)
        reqtype = "ignition.msgs.Pose" 
        reptype = "ignition.msgs.Boolean" 

        # configure update loop
        now_s = time.time()
        start_time_s = now_s

        # update_rate_hz = 1.0
        update_period_s = 1.0 / update_rate_hz
        last_update_time_s = now_s

        sim_time_s = now_s - start_time_s

        while True:
            now_s = time.time()
            time_s = now_s - start_time_s
            if now_s - last_update_time_s >= update_period_s:
                last_update_time_s = now_s

                # check we have valid data

                # create request message
                vector3d_msg = Vector3d()
                vector3d_msg.x = 0.01 + pose_offset_x
                vector3d_msg.y = 0.01+ pose_offset_y
                vector3d_msg.z =  pose_offset_z

                quat_msg = Quaternion()

                pose_msg = Pose()
                pose_msg.name = model_name
                pose_msg.position.CopyFrom(vector3d_msg)
                pose_msg.orientation.CopyFrom(quat_msg)

                # submit request (blocking)
                result = node.request(service, pose_msg, timeout_ms, reptype)

                if not result:
                    print("[{:.1f}] update failed".format(time_s))

    except KeyboardInterrupt:
        pass